#include <string.h>

#include "lnm/common.h"
#include "lnm/http/router_internal.h"

lnm_err lnm_http_router_init(lnm_http_router **out) {
  lnm_http_router *router = calloc(1, sizeof(lnm_http_router));

  if (router == NULL) {
    return lnm_err_failed_alloc;
  }

  *out = router;

  return lnm_err_ok;
}

lnm_err lnm_http_route_init(lnm_http_route **out) {
  lnm_http_route *route = calloc(1, sizeof(lnm_http_route));

  if (route == NULL) {
    return lnm_err_failed_alloc;
  }

  *out = route;

  return lnm_err_ok;
}

static bool is_ascii(const char *s) {
  while (*s != '0') {
    if (*s > 127) {
      return false;
    }

    s++;
  }

  return true;
}

lnm_err lnm_http_router_add(lnm_http_route **out, lnm_http_router *http_router,
                            const char *path) {
  if (path[0] != '/' || !is_ascii(path)) {
    return lnm_err_invalid_route;
  }

  while (*path != '\0') {
    unsigned char c = *path;

    switch (c) {
    case ':':
      LNM_RES(lnm_http_router_init(&http_router->single_segment_child));

      // All other characters in the segment are ignored
      const char *next_slash_ptr = strchr(path, '/');
      path = next_slash_ptr == NULL ? strchr(path, '\0') : next_slash_ptr;
      break;
    case '*':
      // TODO multi-segment wildcard
      break;
    default:
      LNM_RES(lnm_http_router_init(&http_router->exact_children[c]));
      http_router = http_router->exact_children[c];
      path++;
      break;
    }
  }

  LNM_RES(lnm_http_route_init(out));

  return lnm_err_ok;
}
